function testBombeTrajectoire(startPos, startDirection, startSpeed)
	local trajectoireGrp = levelGrp:FindByName("trajectoire")
	
	if trajectoireGrp == nil then
		trajectoireGrp = levelGrp:CreateGroup("trajectoire")
		trajectoireGrp:SetZpos(10)
	end
	
	local t = 0
	
	local collision = false
	local lineId = 1
	
	while (not collision) and (t < 3) do
		local nextT = t + trajectoireTestTimeStep
		
		
		local pos1 = getTrajectoirPos(startPos, startDirection, startSpeed, t)
		local pos2 = getTrajectoirPos(startPos, startDirection, startSpeed, nextT)
		
		local tbl = bombeCollisionTest(pos1, pos2)
		if #tbl ~= 0 then
			collision = true
			pos2 = tbl[1].point
		end
		createTrajectoireLine(pos1, pos2, trajectoireGrp, lineId)
		t = nextT
		
		-- Lineid++
		lineId = lineId + 1
	end
	
	-- Destroy the not used particle lines
	local tmp = true
	while tmp ~= nil do
		local objName = "particle_" .. tostring(lineId)
		tmp = trajectoireGrp:FindByName(objName)
		
		if tmp ~= nil then
			tmp:Destroy()
		end
		
		-- Lineid++
		lineId = lineId + 1
	end
end

function clearBombeTrajectoire()
	local trajectoireGrp = levelGrp:FindByName("trajectoire")
	
	if trajectoireGrp ~= nil then
		trajectoireGrp:DestroyChildren()
	end
end

function bombeCollisionTest(pos1, pos2)	
	local width = (pos2 - pos1):GetSize()
	local height = bombeRadius
	
	local angle = (pos2 - pos1):GetAngle()
	local centerPos = (pos2 - pos1) + pos1
	
	local ph = gameScreen:CreatePhShape(phGroupNone, "ph")
	ph:AddPoint(Vector2D(- width / 2, - height / 2))
	ph:AddPoint(Vector2D(width / 2, - height / 2))
	ph:AddPoint(Vector2D(width / 2, height / 2))
	ph:AddPoint(Vector2D(- width / 2, height / 2))
	ph:SetGlobalPosition(centerPos)
	ph:SetRotation(angle)
	
	local tbl = ph:IsIntersectWithGroup(phGroupDesign)
	ph:Destroy()
	
	for i, struct in pairs(tbl) do	
		if UserDataExistsAndIsEqual(struct.phObj, "_isBombePass", true) then -- Pas de collision avec la bombe
			table.remove(tbl, i)
		end
	end
	
	return tbl
end

function getTrajectoirPos(startPos, startDirection, startSpeed, t)
	return startPos + Vector2D(t * startDirection.x * startSpeed, 0.5 * gravity * t * t + t * startDirection.y * startSpeed)
end

function createTrajectoireLine(pos1, pos2, group, lineId)
	-- Simple line
	--groupPos = group:GetGlobalPosition()
	--local line = group:CreateShapeLine("l", pos1 - groupPos, pos2 - groupPos, 2, Color(255, 0, 0), 0, Color(0, 0, 0))
	
	-- Particles
	local groupPos = group:GetGlobalPosition()
	
	local objName = "particle_" .. tostring(lineId)
	local particleObj = group:FindByName(objName)
	
	if particleObj ~= nil then -- We reuse
		local emitter = ParticleEmitter()
		emitter:MakeLine(pos1 - groupPos, pos2 - groupPos)
		emitter:SetAngles(3.14 * 2, 0)
		
		particleObj:SetEmitter(emitter)
		particleObj:SetFlow(3)
		particleObj:SetForce(0, 30)
		particleObj.position = pos1 - groupPos
	else
		-- Particle
		local model = ParticleModel(0.4, 0.8)
		model:AddMutableParam("alpha", 0.7, 0)
		model:AddRandomParam("blue", 0.6, 1)
		model:AddRandomParam("red", 0, 0.4)
		model:AddRandomParam("green", 0.5, 0.7)

		local emitter = ParticleEmitter()
		emitter:MakeLine(pos1 - groupPos, pos2 - groupPos)
		emitter:SetAngles(3.14 * 2, 0)

		-- Particle
		local particleObj = group:CreateParticleSource(objName, model, 220)
		particleObj:SetEmitter(emitter)
		particleObj:SetFlow(3)
		particleObj:SetForce(3, 30)
		particleObj:MakeRenderPoints(9 + 1 * lineId)
		--particleObj:MakeRenderLines(1, 200000)
		--particleObj:SetGravity(Vector2D(0, -30))
		
		particleObj.position = pos1 - groupPos
	end
end
