
------------------------------------------------------------
-- Map base
------------------------------------------------------------

mapGrp = nil
mapData = {}
mapConfig = {}
mapLoadFn = nil
canAccessAllLevel = false

function loadMapFromFile(filename, gameMode) -- Load a level but to not create a checkpoint
	dofile("scripts/maps/" .. filename)
	loadMap(gameMode)
end


function loadMap(gameMode) -- Load a level but to not create a checkpoint	
	if gameMode ~= "campagne" and gameMode ~= "challenge" then
		printConsoleError("Can't load map : gameMode '" .. gameMode .. "' unknown")
		return
	end

	-- Remove level data
	mapData = {}
	
	-- Remove level config
	mapConfig = {}
	
	-- gameMode
	mapData.gameMode = gameMode
	
	-- Load the level data
	initCurrentMapData()

	-- Doit rester une variable globale
	mapGrp = levelMenuScreen:FindByName("mapGrp")
	
	-- Remove last level + the data added to the group
	loadMapObjects()
end

function loadMapObjects() -- Load / reload the objects
	-- Remove level config
	mapConfig = {}

	-- Remove last level + the data added to the group
	if mapGrp ~= nil then
		mapGrp:Destroy()
	end
	mapGrp = levelMenuScreen:CreateGroup("mapGrp")
	
	-- Load new level
	createCurrentMap(mapGrp)
	
	-- Dep lines
	createDepLines(mapGrp)
	
	-- Tooltip
	mapToolTipInit()
	
	
	-- Elem de base
	btSize = texMenuBtNormal:GetSize()
	local bt = mapGrp:CreateGroup("levelBt")
	buttonControlCreate(bt, _("Return to Menu"), btSize)
	buttonSetEventFunction(bt, function() myChangeScreen(mainMenuScreen, true, "slide2"); end)
	buttonSetFont(bt, 34, Color(255, 255, 255), fontFixedsys)
	buttonSetImagePack(bt, texMenuBtNormal, texMenuBtOver, texMenuBtPressed)
	bt:SetPosition(screenSize - btSize - Vector2D(10, 10))
	
	-- Info
	local text = mapGrp:CreateText("modeTxt", "Mode " .. mapData.gameMode, 34, Color(255, 255, 255), Vector2D(30, screenSize.y - 34 - 30))
	text:SetFont(fontFixedsys)
end

------------------------------------------------------------
-- Tool tip
------------------------------------------------------------
function mapToolTipInit()
	local grp = mapGrp:CreateGroup("toolTipGrp", Vector2D(0, 0))
	grp:SetZpos(1000)
	
	-- Text
	local tmp = grp:CreateText("text", "", 40, Color(255, 255, 255), Vector2D(10, 7))
	tmp:SetMaxWidth(600)
	
	-- Color rect
	grp:CreateShape("backRect")
end

function mapToolTipSetText(text)
	local grp = mapGrp:FindByNameGroup("toolTipGrp")
	
	-- Text
	local tmp = grp:FindByNameText("text")
	tmp:SetText(text)
	local txtSize = tmp:GetTextSize()
	
	-- Resize
	local windowSize = txtSize + Vector2D(20, 20)
	local tmp = grp:FindByNameShape("backRect")
	tmp:MakeRectangle(Vector2D(0, 0), windowSize, Color(0, 0, 0, 210), 4, Color(157, 135, 13))
	tmp:SetZpos(-100)
end

function mapToolTipSetPosition(position)
	local grp = mapGrp:FindByNameGroup("toolTipGrp")
	local screen = mapGrp:GetParentScreen()
	
	position = position - screen:GetGlobalPosition()
	
	local rect = grp:FindByNameShape("backRect")
	local rectSize = rect:GetPointPosition(3) - rect:GetPointPosition(1)
	
	-- Anti dépassement
	if (position.x + rectSize.x) > screenSize.x then
		position.x = position.x - rectSize.x
	end
	
	if (position.y + rectSize.y) > screenSize.y then
		position.y = position.y - rectSize.y
	end
	
	grp:SetPosition(position + mapGrp:GetGlobalPosition() - screen:GetGlobalPosition())
end

function mapToolTipShow(isShow)
	local grp = mapGrp:FindByNameGroup("toolTipGrp")
	
	grp:Show(isShow)
	
	-- Mouse event
	if isShow then
		if not UserDataExistsAndIsNotEqual(grp, "mouseEvt", nil) then -- Evt pas encore créé
			local evt = grp:GetParentScreen():RegisterMouseMoveEvent(function(position, evt)
																	mapToolTipSetPosition(position)
																end)
			guiLib_addAutoDestroyEvent(grp, evt)
			grp:SetUserData("mouseEvt", evt)
		end
	else
		if UserDataExistsAndIsNotEqual(grp, "mouseEvt", nil) then -- Evt créé => on delete
			local evt = grp:GetUserData("mouseEvt")
			guiLib_removeAutoDestroyEvent(grp, evt)
			grp:SetUserData("mouseEvt", nil)
			evt:Destroy()
		end
	end
end

function getMedaillesTimes(baseChallengeTime)
	local str = _("Rewards :")
	str = str .. "\n - " .. _("Gold") .. " : " .. lvlProgress.priv.timeFormat(baseChallengeTime * goldRankRatio)
	str = str .. "\n - " .. _("Silver") .. " : " .. lvlProgress.priv.timeFormat(baseChallengeTime * silverRankRatio)
	str = str .. "\n - " .. _("Bronze") .. " : " .. lvlProgress.priv.timeFormat(baseChallengeTime * bronzeRankRatio)
	return str
end

function getLevelInfoText(grp, baseChallengeTime)
	local lvlName = grp:GetUserData("levelFile")
	local humanName = grp:GetUserData("humanName")
	local status = grp:GetUserData("status")
	local baseChallengeTime = grp:GetUserData("baseChallengeTime")

	str = humanName .. " :\n\n"
	
	if status == "todo" then
		str = str .. _("You can do that mission !")
		
		if isMapChallenge() then
			str = str .. "\n\n" .. getMedaillesTimes(baseChallengeTime)
		end
	elseif status == "done" then
		str = str .. lvlProgress.getStatsText(lvlProgress.getLevelStats(lvlName, isMapChallenge()), true, isMapChallenge(), baseChallengeTime)
		
		if isMapChallenge() then
			str = str .. "\n\n" .. getMedaillesTimes(baseChallengeTime)
		end
	else
		str = str .. _("You can't do that mission yet.")
	end
	
	return str
end

------------------------------------------------------------
-- Base
------------------------------------------------------------
function getIsoBarycentre(coordTbl)
	local coord = Vector2D(0, 0)
	
	for i = 1, #coordTbl do
		coord = coord + coordTbl[i]
	end
	
	return coord / #coordTbl
end

function isMapChallenge()
	return mapData.gameMode == "challenge"
end

------------------------------------------------------------
-- Level bt
------------------------------------------------------------
function createLevelBt(parentGrp, phPointTbl, imgPos, overTex, levelFile, humanName, depList, baseChallengeTime)
	local grp = createMapBt(parentGrp, phPointTbl, imgPos, overTex,
							function(grp)
								if (grp:GetUserData("status") ~= "noAccess") or (canAccessAllLevel == true) then
									loadLevelFromFile(grp:GetUserData("levelFile"), mapData.gameMode, grp:GetUserData("baseChallengeTime"))
									--myChangeScreen(gameScreen, false)
									myChangeScreen(gameScreen, true, "fade", gameRenderScreen)
								else
									popupDialog(parentGrp:GetParentScreen(), "dialogOk", _("You can't access that level yet !"), function() end, nil)
									soundNs.PlayGlobalSnd(sndLevelNoAccess)
								end
							end,
							
							function(grp, isInside)
								-- Tooltip
								if isInside then
									mapToolTipSetText(getLevelInfoText(grp, grp:GetUserData("baseChallengeTime")))
								end
								mapToolTipShow(isInside)
								mapToolTipSetPosition(GetMousePosition())
							end)
	grp:SetUserData("levelFile", levelFile)
	grp:SetUserData("humanName", humanName)
	grp:SetUserData("depList", depList)
	grp:SetUserData("baseChallengeTime", baseChallengeTime)
	
	-- Info icon
	local pos = getIsoBarycentre(phPointTbl)
	grp:SetUserData("isoBarycentre", pos)
	
	local tex = levelToDoTex
	local status = "todo"
	
	if lvlProgress.isLevelDone(levelFile, isMapChallenge()) then
		tex = levelDoneTex
		status = "done"
	end
	
	-- Dep
	local isOk = true
	if status == "todo" then
		for i = 1, #depList do
			if not lvlProgress.isLevelDone(depList[i], isMapChallenge()) then
				isOk = false
				break
			end
		end
	end
	
	if not isOk then
		tex = levelNoAccessTex
		status = "noAccess"
	end
	
	grp:SetUserData("status", status)
	local tmp = parentGrp:CreateSprite("icon", tex, pos - tex:GetSize() / 2)
	tmp:SetZpos(200)
	
	-- Save
	mapConfig.levelList = mapConfig.levelList or {}
	mapConfig.levelList[levelFile] = grp
	
	-- Return
	return grp
end

------------------------------------------------------------
-- Create dep lines
------------------------------------------------------------
function createDepLines(parentGrp)
	for levelName, grp in pairs(mapConfig.levelList) do	
		local depList = grp:GetUserData("depList")
		
		for i = 1, #depList do
			local pos1 = grp:GetUserData("isoBarycentre")
			
			local grp2 = mapConfig.levelList[depList[i]]
			local pos2 = grp2:GetUserData("isoBarycentre")
			
			-- Line
			local isDone = grp2:GetUserData("status") == "done"
			local color =  Color(255, 0, 0, 70)
			
			if isDone then
				color =  Color(0, 255, 0, 70)
			end
		
			local tmp = parentGrp:CreateShapeLine("depLine", pos1, pos2, 10, color, 0, Color(0, 0, 0))
		end
	end
end

------------------------------------------------------------
-- new map bt
------------------------------------------------------------
function createNewMapBt(parentGrp, phPointTbl, imgPos, overTex, mapFile)
	local grp = createMapBt(parentGrp, phPointTbl, imgPos, overTex,
								function(grp)
									myChangeScreen(gameScreen, true, "fade", gameRenderScreen, {onHalfFn = function() loadMapFromFile(grp:GetUserData("mapFile")); end})
								end,

								function(grp)
									
								end)

	grp:SetUserData("mapFile", mapFile)
	
	-- Return
	return grp
end

------------------------------------------------------------
-- Map bt (base)
-- fn arg : (bt : Group)
------------------------------------------------------------

function createMapBt(parentGrp, phPointTbl, imgPos, overTex, fn, fnOver)
	local grp = parentGrp:CreateGroup("levelBt")
	grp:SetUserData("isEnabled", true)
	grp:SetUserData("fn", fn)
	grp:SetUserData("fnOver", fnOver)
	
	-- Ph
	local ph = grp:CreatePhShape(phGroupNone, "ph")
	for i = 1, #phPointTbl do
		ph:AddPoint(phPointTbl[i])
	end
	
	-- img
	local tmp = grp:CreateSprite("overSpr", overTex, imgPos)
	tmp:Show(false)
	
	-- Events
	local zoneEvent = grp:GetParentScreen():RegisterMouseZoneEvent(privateLevelBtOver, ph)
	zoneEvent:SetUserData("grp", grp)
	guiLib_addAutoDestroyEvent(grp, zoneEvent)
	
	local event = grp:GetParentScreen():RegisterButtonInZoneEvent(privateLevelBtClick, zoneEvent, "MouseLeft", true)
	guiLib_addAutoDestroyEvent(grp, event)
	
	-- Return
	return grp
end

function privateLevelBtOver(event)
	local grp = event:GetUserData("grp")
	local spr = grp:FindByName("overSpr")

	spr:Show(event:IsInside())

	-- Event
	grp:GetUserData("fnOver")(grp, event:IsInside())
end

function privateLevelBtClick(key, pressed, zoneEvent, event)
	local grp = zoneEvent:GetUserData("grp")
	grp:GetUserData("fn")(grp)
end
