------------------------------------------------------------
-- Public
------------------------------------------------------------
musicNs = musicNs or {}

function musicSetVolume(volume)
	musicNs.volume = volume

	if musicNs.currentMusic ~= nil then
		musicNs.currentMusic:SetVolume(musicNs.volume)
	end
end

function musicSetPlaylist(fileList, isRandom)
	-- Table
	if type(fileList) ~= "table" then
		fileList = {fileList}
	end
	
	-- Evts
	if musicNs.endMusicTimer ~= nil then
		musicNs.endMusicTimer:Destroy()
		musicNs.endMusicTimer = nil
	end
	
	-- fileList
	musicNs.currentPlaylist = fileList
	
	-- Random
	musicNs.isRandom = (isRandom == true)
	
	-- Start a music
	musicNs.startMusicEndTime(nil)
end

function musicStop(duration)
	local startRatio = 0

	-- Already changing
	if musicNs.changeMusicChronoEvt ~= nil then
		-- The next music is now the current
		musicNs.currentMusic = musicNs.currentMusicNext
		musicNs.currentMusicNext = nil
		
		-- Really destroy
		collectgarbage()
		
		startRatio = musicNs.changeMusicChronoEvt:GetRatio()
	end

	if musicNs.currentMusic ~= nil then
		musicNs.createChangeMusicAnim(musicNs.currentMusic, nil)
	end
	
	if duration ~= nil then
		musicNs.changeMusicChronoEvt:SetTotalTime(duration)
	end
	
	musicNs.changeMusicChronoEvt:SetRatio(startRatio)
end

------------------------------------------------------------
-- Defined playlist
------------------------------------------------------------

function musicNs.loadPlayListMenu()
	musicSetPlaylist({"Turn_turn_around_Ecran_titre.ogg"}, true)
end

function musicNs.loadPlayListLevel(lvlName)
	local playList = nil
	
	local tblLvl1to5 = {"level01.lua", "level02.lua", "level03.lua", "level04.lua", "level05.lua"}
	local tblLvl6ABC = {"level06.lua", "levelA.lua", "levelB.lua", "levelC.lua"}
	
	if lvlName == "level00.lua" then
		playList = {"PIXEL_MY_HEART.ogg"}
	elseif table.isInTable(tblLvl1to5, lvlName) ~= nil then
		playList = {"Turn_turn_around_Ecran_titre.ogg", "Old_memories_Niveau_3_-_Partie_1.ogg", "NEVER_BORN.ogg"}
	elseif table.isInTable(tblLvl6ABC, lvlName) ~= nil then
		playList = {"The_Great_Paper_Adventure_Niveau_final.ogg", "Supernova_Boss_final.ogg", "Little_blue_rabbit_Niveau_2_-_Partie_1.ogg"}
	end
	
	if playList ~= nil then
		musicSetPlaylist(playList, true)
	else
		printErrorConsole("musicNs.loadPlayListLevel(lvlName) : unknown level '" .. lvlName .. "' !")
	end
end

------------------------------------------------------------
-- Private
------------------------------------------------------------

------------------------------------------------------------
-- Data
------------------------------------------------------------

-- Base
musicNs.volume = 100
musicNs.currentMusic = nil
musicNs.currentMusicNext = nil
musicNs.currentMusicId = nil
musicNs.isRandom = true
musicNs.currentPlaylist = {}

-- Evt
musicNs.changeMusicChronoEvt = nil
musicNs.endMusicTimer = nil

------------------------------------------------------------
-- Music end timer
------------------------------------------------------------

-- Fin de music -> Changement
function musicNs.startMusicEndTime(timerEvt)
	-- Fn called without evt
	if timerEvt ~= nil then
		if musicNs.endMusicTimer ~= timerEvt then
			printErrorConsole("musicNs.startMusicEndTime() : Error: Two different events !")
		end
	
		timerEvt:Destroy()
		musicNs.endMusicTimer = nil
	end
	
	-- Select new music
	local fileList = musicNs.currentPlaylist
	
	if (musicNs.currentMusicId ~= nil) and (musicNs.currentMusicId > #fileList) then -- musicNs.currentMusicId pas dans le tableau (cas ou le tableau est devenu plus petit)
		musicNs.currentMusicId = nil
	end
	
	if musicNs.currentMusicId == nil then -- pas de musique avant
		if musicNs.isRandom then
			musicNs.currentMusicId = math.random(#fileList)
		else
			musicNs.currentMusicId = 1 -- We start with the first
		end
			
		local newMusic = Music("music/" .. fileList[musicNs.currentMusicId])
		musicNs.createChangeMusicAnim(musicNs.currentMusic, newMusic)
		
		--print("New : " .. fileList[musicNs.currentMusicId])
	else
		if #fileList == 1 then -- Une seule musique
			local newMusic = Music("music/" .. fileList[1])
			musicNs.currentMusicId = 1
			musicNs.createChangeMusicAnim(musicNs.currentMusic, newMusic)
			
			--print("Single : " .. fileList[musicNs.currentMusicId])
		else -- Plusieurs musiques => On prend pas la précedente
			local newMusicId
			if musicNs.isRandom then -- random
				newMusicId = math.random(#fileList - 1) -- -1 pour passer la musique qu'on veut passer
				if newMusicId == musicNs.currentMusicId then -- Si on tombe sur l'ancienne
					newMusicId = #fileList -- on prend la dernière de la liste
				end
			else
				newMusicId = musicNs.currentMusicId + 1 -- -1 pour passer la musique qu'on veut passer
				if newMusicId > #fileList then
					newMusicId = 1
				end
			end
			
			local newMusic = Music("music/" .. fileList[newMusicId])
			musicNs.currentMusicId = newMusicId
			musicNs.createChangeMusicAnim(musicNs.currentMusic, newMusic)
			
			--print("Random : " .. fileList[musicNs.currentMusicId])
		end
	end
	
	-- Start next timer
	musicNs.endMusicTimer = GetCurrentScreen():RegisterTimer(musicNs.startMusicEndTime, musicNs.currentMusicNext:GetDuration() - musicTransitionTime, true)
end

------------------------------------------------------------
-- Migrate events to a new screen
------------------------------------------------------------

function musicNs.changeScreen(newScreen)
	-- Recreate events
	if musicNs.endMusicTimer ~= nil then
		local newEvt = newScreen:RegisterTimer(musicNs.endMusicTimer:GetLuaFunction(), musicNs.endMusicTimer:GetInterval(), true)
		newEvt:SetElapsedTime(musicNs.endMusicTimer:GetElapsedTime())
		
		musicNs.endMusicTimer:Destroy()
		musicNs.endMusicTimer = newEvt
	end

	if musicNs.changeMusicChronoEvt ~= nil then
		local newEvt = newScreen:RegisterChronometer(musicNs.changeMusicChronoEvt:GetLuaFunction(), musicNs.changeMusicChronoEvt:GetTotalTime(), true)
		newEvt:SetRatio(musicNs.changeMusicChronoEvt:GetRatio())
		
		musicNs.changeMusicChronoEvt:Destroy()
		musicNs.changeMusicChronoEvt = newEvt
	end
end

------------------------------------------------------------
-- Change the music with a fade
------------------------------------------------------------

function musicNs.onChangeMusicEvt(ratio, evt)
	if musicNs.currentMusic ~= nil then
		musicNs.currentMusic:SetVolume((1 - ratio) * musicNs.volume)
	end
	
	if musicNs.currentMusicNext ~= nil then
		musicNs.currentMusicNext:SetVolume(ratio * musicNs.volume)
	end
	
	if ratio == 1 then
		musicNs.currentMusic = nil
			
		-- Really destroy the music
		collectgarbage()
		
		musicNs.currentMusic = musicNs.currentMusicNext
		musicNs.currentMusicNext = nil
		
		-- Destroy evt
		if musicNs.changeMusicChronoEvt ~= evt then
			printErrorConsole("musicNs.startMusicEndTime() : Error: Two different events !")
		end
		
		musicNs.changeMusicChronoEvt:Destroy()
		musicNs.changeMusicChronoEvt = nil
	end
end

------------------------------------------------------------
-- createChangeMusicAnim
------------------------------------------------------------

function musicNs.createChangeMusicAnim(musicBack, musicNew)
	-- Destroy previous
	if musicNs.changeMusicChronoEvt ~= nil then
		musicNs.changeMusicChronoEvt:Destroy()
		musicNs.changeMusicChronoEvt = nil
	end

	musicNs.currentMusic = musicBack
	musicNs.currentMusicNext = musicNew
	
	-- Play
	if (musicNs.currentMusic ~= nil) and (not musicNs.currentMusic:IsPlaying()) then
		musicNs.currentMusic:Play()
	end
	
	if (musicNs.currentMusicNext ~= nil) and (not musicNs.currentMusicNext:IsPlaying()) then
		musicNs.currentMusicNext:Play()
	end
	
	-- Start new chrono
	musicNs.changeMusicChronoEvt = GetCurrentScreen():RegisterChronometer(musicNs.onChangeMusicEvt, musicTransitionTime, true)
end
