------------------------------------------------------------
-- Physic collision events
------------------------------------------------------------

function onPlayerCorpsColision(playerPh, sol, pos, penetrationVector)
	local frameTime = GetFrameTime()
	
	local player = playerPh:GetParent()
	
	if (isPlayer(player)) and (sol:GetParent():GetName() ~= "bombeGrp") then
		if isButton(sol) then -- Buttons
			onPlayerButtonColision(playerPh, sol, pos, penetrationVector)
			return
		end
		
		if UserDataExistsAndIsEqual(sol, "_isHurtZone", true) then -- Zone qui tue
			if isLevelChallenge() then
				timeAddMalus(frameTime * timeMalusRatioHurtZone)
				player:FindByName("spr"):SetColor(Color(255, 0, 0))
				player:SetUserData("frameIdColor", GetUniqueFrameId())
				return -- No collision
			else
				onPlayerKilled(player, "hurtZone")
				return
			end
		end
	
		if UserDataExistsAndIsEqual(sol, "_isPlayerPass", true) then -- Pas de collision avec le joueur
			return
		end
	
		-- Faire reculer l'objet.
		player:Move(penetrationVector)
		
		-- Prendre droite du choc
		-- Prendre la normale à cette droite
		-- Supprimer cette composante normale du vecteur vitesse de l'objet
		local speed = player:GetSpeed()
		local penetrationDirection = penetrationVector:GetDirection()
		local penetrationSpeedNorm = speed:GetDotProduct(penetrationDirection)
		
		-- Test si on créve sur le mur
		crashSpeed = penetrationDirection:GetDotProduct(speed)
		if math.abs(crashSpeed) >= lethalMinSpeed then
			if isLevelChallenge() then
				timeAddMalus((math.abs(crashSpeed) - lethalMinSpeed) * timeMalusRatioCrash)
				
				-- Red
				player:FindByName("spr"):SetColor(Color(255, 0, 0))
				player:SetUserData("frameIdColor", GetUniqueFrameId())
				
				-- Snd
				soundNs.PlayGlobalSnd(sndManDie)
			else
				onPlayerKilled(player, "crash")				
			end
		end
		
		-- Si la vitesse est déjà dans la direction du dégagement (peut arriver sur le coin des objets) => On ne change pas la vitesse
		if penetrationSpeedNorm < 0 then
			speed = speed - penetrationDirection * penetrationSpeedNorm
		end
			
		
		--  => + ajout rebond selon la composante normale retranchée, dans l'autre sens ?
		-- Réduire la nouvelle vitesse de l'objet (frottement) -> en fonction de la vitesse d'arrivée sur la plate forme ("N")
	
		local frictionDirection = penetrationDirection:GetDirectNormal()
		
			-- La friction n'agit que sur les surfaces horizontales
		frictionDirection = Vector2D(1, 0) * frictionDirection:GetDotProduct(Vector2D(1, 0))
		if frictionDirection ~= Vector2D(0, 0) then
			local removedFrictionSpeed = frictionDirection * speed:GetDotProduct(frictionDirection) * friction * GetFrameTime()
			speed = speed  - removedFrictionSpeed
		end
		
		-- Application de la vitesse
		player:SetSpeed(speed)
		
		-- On enléve de la position la vitesse retranchée pour que la vitesse représente bien le mvt durant la frame
		--player:Move(-1 * removedFrictionSpeed * GetFrameTime())
		
		-- On saute plus
		-- Si vecteur de penétration vers le haut => On saute plus
		if penetrationVector.y < 0 then
			player:SetUserData("isOnAir", false)
		end
		
		if math.abs(player:GetSpeed():GetSquareSum()) < 0.1 then -- < 0.05 => O pour arreter la gravité
			player:SetSpeed(Vector2D(0, 0))
		end
		
		if (player:GetSpeed() == Vector2D(0, 0)) and (penetrationVector.y < 0) then -- Si la vitesse et nulle et qu'on ne se fait pas jeter d'une collision avec le haut d'une plate forme
			startGravity(player, false)
			onPlayerMove(player) -- Move camera to the current position
		end
	end
end


function onImposeGravity(frameTime, event)
	local body = event:GetUserData("body")
	
	-- Is jumping
	if body:GetSpeed().y ~= 0 then -- Si la vitesse verticale est déjà différente de 0 => On saute / on tombe 
		body:SetUserData("isOnAir", true)
	end
	
	body:SetSpeed(Vector2D(body:GetSpeed().x, body:GetSpeed().y + gravity * frameTime))
	
	if body:UserDataExists("keyTbl") then -- Player
		onPlayerMove(body)
	end
end

------------------------------------------------------------
-- Physic utils
------------------------------------------------------------

-- Utils
function startGravity(body, isOn)
	if (isOn and isIdle(body)) then
		local event = body:GetParentScreen():RegisterFrameEvent(onImposeGravity)
		guiLib_addAutoDestroyEvent(body, event)
		event:SetUserData("body", body)
		
		body:SetUserData("gravityFn", event)
	elseif not isOn and not isIdle(body) then
		guiLib_removeAutoDestroyEvent(body, body:GetUserData("gravityFn"))
		body:GetUserData("gravityFn"):Destroy()
		body:SetUserData("gravityFn", nil)
	end
end

function isIdle(body)
	return not(body:UserDataExists("gravityFn") and (body:GetUserData("gravityFn") ~= nil))
end

function isPlayer(playerGrp)
	return playerGrp:UserDataExists("keyTbl")
end

function isButton(phObj)
	return phObj:UserDataExists("_btFnOnTouch")
end

------------------------------------------------------------
-- Force gravity for all players (when a physic wall is removed for example)
------------------------------------------------------------

function forcePlayersGravity()
	for i = 1, #levelPlayerTbl do
		startGravity(levelPlayerTbl[i], true)
	end
end
